# IntroScripts.R
# Scripts for the Intro to R webinar, 2018.
# copyright Practical Stats LLC, 2018.  See practicalstats.com


# BOOTLCL  
# Computes bootstrapped estimates of the LCL95.
# Written by Ed Gilroy, 2015.

bootLCL <- function(x1,conf= 95, R =10000){
confX = conf
cat("Bootstrap Estimate of an Lower Confidence Limit ",'\n') 
xname = deparse(substitute(x1))
x1 = -x1
 cat("           of the Mean of ",xname,'\n')
cat('\n')

 n1 <- length(x1)
conf = conf/100
dbar <-  mean(x1)
z<- numeric(R)
  for (i in 1:R){
  mn <- sample(n1,n1,replace = TRUE)
   dbardash <- mean(x1[mn])
   z[i] <- dbardash }
 pvalup <- sum(z > abs(dbar))/R
 pvaldn <- sum(z < -abs(dbar))/R
 pval <- pvaldn + pvalup
z <- sort(z)
i95 <- conf*R
i95 <- round(i95)
i95
lcl95 <- -z[i95]

conf <- sum(z >  lcl95)/R
conf <- 100.0*conf

dbar = -dbar
RESULTS <- data.frame(LCL = lcl95, XBAR = dbar, CONF=confX,NREPS = R)
print (RESULTS)
z= -z
 plot(density(z), main = paste("Bootstrap density ",confX,"%LCL of mean",xname),xlab = R,sub = ("Bootstrap estimates"),yaxs= "i")
 abline(v = lcl95)
 abline(v = dbar, col ="red")

 hist(z,main = paste("Bootstrap ",confX,"%LCL of mean",xname),xlab = R,sub = ("Bootstrap estimates"))
 abline(v = lcl95)
abline(v = dbar, col ="red")
}



# PERM1SAMPLE 1.4
#  computes a permutation test for the one-sample t-test
#  default R =10000
#  by D.R. Helsel.  Aug 5, 2015.
# to reference the use of this script, cite  Helsel, D.R. (2015), perm1sample
# script for R.  Practicalstats.com.  May be obtained by emailing a request to
# dhelsel@practicalstats.com

perm1sample <- function(dat,R=10000,conf=95,alternative="two.sided"){
dat1=na.omit(dat)
diffObs <- mean(dat1)
conf=conf/100.0
#print(diffObs)

aname = deparse(substitute(dat))
#print(aname)
resampMeanDiff <- numeric(R)
for (i in 1:R) {
       signs <- sample( c(1,-1),length(dat1), replace = T)
       resamp <- dat1 * signs
       resampMeanDiff[i] <- mean(resamp)
       }

highprob <- length(resampMeanDiff[resampMeanDiff >= diffObs])/R
lowprob <- length(resampMeanDiff[resampMeanDiff <= (-1)*diffObs])/R
# print(highprob)
# print(lowprob)
prob2tailed <- lowprob + highprob
if (diffObs<0) {prob2tailed<- (1-lowprob)+(1-highprob)}
resampMD <- sort(resampMeanDiff)

prob=prob2tailed
if (alternative== "greater"|alternative=="g") prob=highprob
if (alternative== "less"|alternative=="l") prob=(1-lowprob)

xs=round(R/200)
xstart=resampMD[xs]
ystart=R/15
ys=ystart*0.85
txtd=paste(aname)
txte=paste0("Mean ",txtd," shown as dashed line")
if (alternative== "two.sided"| alternative=="t"|alternative=="two") {txtc=" "} else {txtc="than zero"}
t.normal<-t.test(dat,alternative=alternative)

# print output
cat("Permutation One-Sample Test",'\n')
cat(txtd," ","alternative =",t.normal$alternative,txtc,'\n',"p-value =",signif(prob,6),'\n','\n')
  # Compare to Student's t
##cat("The normal-theory one-sample t test is:",'\n')
##cat("t= ",t.normal$statistic,"  p= ",t.normal$p.value,'\n')
##cat("mean =",t.normal$estimate," 95% CI = (",t.normal$conf.int,")",'\n')

# plot results
negt=(-1)*t.normal$estimate
hist(resampMeanDiff, main = "Bars Represent Null Hypothesis",xlab = txte, col="light gray")
abline(v = t.normal$estimate,lty=2,col="purple")
if (alternative=="two.sided"|alternative=="t") abline(v = negt,lty=2,col="purple")
text(xstart,ystart,"p-value")
text(xstart,ys, signif(prob,4))
}


cat(" R scripts loaded for Intro to R webinar.  Many other scripts are available",'\n')
cat(" by taking Practical Stats courses.  See www.practicalstats.com/training/ ",'\n')
  

